@@ -325,7 +325,7 @@ typedef union YYSTYPE
 {
 
 /* Line 2068 of yacc.c  */
-#line 1407 "parser.y"
+#line 1298 "parser.y"
 
   char  *id;
   List  *bases;
@@ -340,7 +340,7 @@ typedef union YYSTYPE
     String *nexcept;
   } dtype;
   struct {
-    char *type;
+    const char *type;
     String *filename;
     int   line;
   } loc;
