@@ -52,6 +52,7 @@
 #define WARN_DEPRECATED_NODEFAULT     123
 #define WARN_DEPRECATED_TYPEMAP_LANG  124
 #define WARN_DEPRECATED_INPUT_FILE    125
+#define WARN_DEPRECATED_NESTED_WORKAROUND 126
 
 /* -- Preprocessor -- */
 
@@ -89,6 +90,12 @@
 #define WARN_PARSE_REC_INHERITANCE    323
 #define WARN_PARSE_NESTED_TEMPLATE    324
 #define WARN_PARSE_NAMED_NESTED_CLASS 325
+#define WARN_PARSE_EXTEND_NAME        326
+
+#define WARN_CPP11_LAMBDA             340
+#define WARN_CPP11_ALIAS_DECLARATION  341
+#define WARN_CPP11_ALIAS_TEMPLATE     342
+#define WARN_CPP11_VARIADIC_TEMPLATE  343
 
 #define WARN_IGNORE_OPERATOR_NEW        350	/* new */
 #define WARN_IGNORE_OPERATOR_DELETE     351	/* delete */
@@ -169,6 +176,8 @@
 #define WARN_TYPEMAP_DIRECTOROUT_PTR   473
 #define WARN_TYPEMAP_OUT_OPTIMAL_IGNORED  474
 #define WARN_TYPEMAP_OUT_OPTIMAL_MULTIPLE 475
+#define WARN_TYPEMAP_INITIALIZER_LIST  476
+#define WARN_TYPEMAP_DIRECTORTHROWS_UNDEF 477
 
 /* -- Fragments -- */
 #define WARN_FRAGMENT_NOT_FOUND       490
@@ -196,6 +205,8 @@
 #define WARN_LANG_TEMPLATE_METHOD_IGNORE 519
 #define WARN_LANG_SMARTPTR_MISSING    520
 #define WARN_LANG_ILLEGAL_DESTRUCTOR  521
+#define WARN_LANG_EXTEND_CONSTRUCTOR  522
+#define WARN_LANG_EXTEND_DESTRUCTOR   523
 
 /* -- Reserved (600-799) -- */
 
@@ -241,6 +252,7 @@
 #define WARN_JAVA_TYPEMAP_JAVACONSTRUCT_UNDEF 823
 #define WARN_JAVA_TYPEMAP_DIRECTORIN_NODESC   824
 #define WARN_JAVA_NO_DIRECTORCONNECT_ATTR     825
+#define WARN_JAVA_NSPACE_WITHOUT_PACKAGE      826
 
 /* please leave 810-829 free for Java */
 
