@@ -11,13 +11,11 @@
  * CHICKEN language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_chicken_cxx[] = "$Id: chicken.cxx 13885 2012-11-14 22:16:07Z wsfulton $";
-
 #include "swigmod.h"
 
 #include <ctype.h>
 
-static const char *usage = (char *) "\
+static const char *usage = "\
 \
 CHICKEN Options (available with -chicken)\n\
      -closprefix <prefix>   - Prepend <prefix> to all clos identifiers\n\
@@ -31,7 +29,7 @@ CHICKEN Options (available with -chicken)\n\
 \n";
 
 static char *module = 0;
-static char *chicken_path = (char *) "chicken";
+static const char *chicken_path = "chicken";
 static int num_methods = 0;
 
 static File *f_begin = 0;
@@ -622,7 +620,7 @@ int CHICKEN::functionWrapper(Node *n) {
       if (in_class)
 	clos_name = NewString(member_name);
       else
-	clos_name = chickenNameMapping(scmname, (char *) "");
+	clos_name = chickenNameMapping(scmname, "");
 
       if (!any_specialized_arg) {
 	method_def = NewString("");
@@ -777,7 +775,7 @@ int CHICKEN::variableWrapper(Node *n) {
       if (in_class)
 	clos_name = NewString(member_name);
       else
-	clos_name = chickenNameMapping(scmname, (char *) "");
+	clos_name = chickenNameMapping(scmname, "");
 
       Node *class_node = classLookup(t);
       String *clos_code = Getattr(n, "tmap:varin:closcode");
@@ -944,7 +942,7 @@ int CHICKEN::constantWrapper(Node *n) {
       if (in_class)
 	clos_name = NewString(member_name);
       else
-	clos_name = chickenNameMapping(scmname, (char *) "");
+	clos_name = chickenNameMapping(scmname, "");
       if (GetFlag(n, "feature:constasvar")) {
 	Printv(clos_methods, "(define ", clos_name, " (", chickenPrimitiveName(scmname), "))\n", NIL);
 	Printv(scm_const_defs, "(set! ", scmname, " (", scmname, "))\n", NIL);
@@ -1374,7 +1372,7 @@ void CHICKEN::dispatchFunction(Node *n) {
       } else if (in_class)
 	clos_name = NewString(member_name);
       else
-	clos_name = chickenNameMapping(scmname, (char *) "");
+	clos_name = chickenNameMapping(scmname, "");
 
       Iterator f;
       List *prev = 0;
