@@ -86,11 +86,11 @@ extern "C"
  you can add new ones here
  (though for now I have not bothered)
 NEW LANGUAGE NOTE:END ************************************************/
-static const char *usage = (char *) "\
+static const char *usage = "\
 Lua Options (available with -lua)\n\
      -elua           - Generates LTR compatible wrappers for smaller devices running elua\n\
      -eluac          - LTR compatible wrappers in \"crass compress\" mode for elua\n\
-     -elua-emulate   - Emulates behaviour of eLua. Usefull only for testing.\n\
+     -elua-emulate   - Emulates behaviour of eLua. Useful only for testing.\n\
                        Incompatible with -elua/-eluac options.\n\
      -nomoduleglobal - Do not register the module name as a global variable \n\
                        but return the module table from calls to require.\n\
@@ -107,16 +107,14 @@ static int elua_ltr = 0;
 static int eluac_ltr = 0;
 static int elua_emulate = 0;
 static int squash_bases = 0;
-/* This  variable defines internal(!) module API level and compatibility options.
- * This variable is controled by -no-old-metatable-bindings option.
- * v2_compatibility - 
+/* The new metatable bindings were introduced in SWIG 3.0.0.
+ * old_metatable_bindings in v2: 
  *                    1. static methods will be put into the scope their respective class
  *                    belongs to as well as into the class scope itself. (only for classes without %nspace given)
  *                    2. The layout in elua mode is somewhat different
  */
-static int v2_compatibility = 0;
-static int v2_compat_names_generation = 1; // This flag can temporaraly disable backward compatible names generation if v2_compatibiliti is enabled
-static const int default_api_level = 2;
+static int old_metatable_bindings = 1;
+static int old_compatible_names = 1; // This flag can temporarily disable backward compatible names generation if old_metatable_bindings is enabled
 
 /* NEW LANGUAGE NOTE:***********************************************
  To add a new language, you need to derive your class from
@@ -207,7 +205,7 @@ public:
   }
 
   /* NEW LANGUAGE NOTE:***********************************************
-     This is called to initalise the system & read any command line args
+     This is called to initialise the system & read any command line args
      most of this is boilerplate code, except the command line args
      which depends upon what args your code supports
      NEW LANGUAGE NOTE:END *********************************************** */
@@ -220,7 +218,6 @@ public:
 
   virtual void main(int argc, char *argv[]) {
 
-    int api_level = default_api_level;	// Default api level
     /* Set location of SWIG library */
     SWIG_library_directory("lua");
 
@@ -240,7 +237,7 @@ public:
 	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i], "-no-old-metatable-bindings") == 0) {
 	  Swig_mark_arg(i);
-	  api_level = 3;
+	  old_metatable_bindings = 0;
 	} else if (strcmp(argv[i], "-squash-bases") == 0) {
 	  Swig_mark_arg(i);
 	  squash_bases = 1;
@@ -256,15 +253,6 @@ public:
       Swig_arg_error();
     }
 
-    // Set API-compatibility options
-    if (api_level <= 2)		// Must be compatible with SWIG 2.*
-      v2_compatibility = 1;
-    // template for further API breaks
-    //if(api_level <= 3)
-    //  v3_compatibility = 1;
-    //if(api_level <= 4)
-    //  v4_compatibility = 1;
-
     // Set elua_ltr if elua_emulate is requested
     if(elua_emulate)
       elua_ltr = 1;
@@ -392,7 +380,7 @@ public:
     Dump(f_header, f_begin);
     Dump(f_wrappers, f_begin);
     Dump(f_initbeforefunc, f_begin);
-    /* for the Lua code it needs to be properly excaped to be added into the C/C++ code */
+    /* for the Lua code it needs to be properly escaped to be added into the C/C++ code */
     escapeCode(s_luacode);
     Printf(f_begin, "const char* SWIG_LUACODE=\n  \"%s\";\n\n", s_luacode);
     Wrapper_pretty_print(f_init, f_begin);
@@ -448,9 +436,9 @@ public:
   /* NEW LANGUAGE NOTE:***********************************************
      This is it!
      you get this one right, and most of your work is done
-     but its going to take soem file to get it working right
+     but its going to take some file to get it working right
      quite a bit of this is generally boilerplate code
-     (or stuff I dont understand)
+     (or stuff I don't understand)
      that which matters will have extra added comments
      NEW LANGUAGE NOTE:END *********************************************** */
   /* ---------------------------------------------------------------------
@@ -546,7 +534,7 @@ public:
     }
 
     /* NEW LANGUAGE NOTE:***********************************************
-       the wrapper object holds all the wrappering code
+       the wrapper object holds all the wrapper code
        we need to add a couple of local variables
        NEW LANGUAGE NOTE:END *********************************************** */
     Wrapper *f = NewWrapper();
@@ -566,7 +554,7 @@ public:
     /* NEW LANGUAGE NOTE:***********************************************
        the format of a lua fn is:
        static int wrap_XXX(lua_State* L){...}
-       this line adds this into the wrappering code
+       this line adds this into the wrapper code
        NEW LANGUAGE NOTE:END *********************************************** */
     Printv(f->def, "static int ", wname, "(lua_State* L) {", NIL);
 
@@ -669,7 +657,7 @@ public:
 	continue;
       } else {
 	/* NEW LANGUAGE NOTE:***********************************************
-	   // why is this code not called when I dont have a typemap?
+	   // why is this code not called when I don't have a typemap?
 	   // instead of giving a warning, no code is generated
 	   NEW LANGUAGE NOTE:END *********************************************** */
 	Swig_warning(WARN_TYPEMAP_IN_UNDEF, input_file, line_number, "Unable to use type %s as a function argument.\n", SwigType_str(pt, 0));
@@ -804,7 +792,7 @@ public:
     Replaceall(f->code, "$result", Swig_cresult_name());
 
     /* Dump the function out */
-    /* in Lua we will not emit the destructor as a wrappered function,
+    /* in Lua we will not emit the destructor as a wrapper function,
        Lua will automatically call the destructor when the object is free'd
        However: you cannot just skip this function as it will not emit
        any custom destructor (using %extend), as you need to call emit_action()
@@ -847,7 +835,7 @@ public:
   /* NEW LANGUAGE NOTE:***********************************************
      This is an extra function used for overloading of functions
      it checks the args & then calls the relevant fn
-     nost of the real work in again typemaps:
+     most of the real work in again typemaps:
      look for %typecheck(SWIG_TYPECHECK_*) in the .swg file
      NEW LANGUAGE NOTE:END *********************************************** */
   int dispatchFunction(Node *n) {
@@ -1041,7 +1029,7 @@ public:
     assert(s_const_tab);
     Printf(s_const_tab, "    %s,\n", constantRecord);
 
-    if ((eluac_ltr || elua_ltr) && v2_compatibility) {
+    if ((eluac_ltr || elua_ltr) && old_metatable_bindings) {
       s_const_tab = Getattr(nspaceHash, "constants");
       assert(s_const_tab);
       Printf(s_const_tab, "    %s,\n", constantRecord);
@@ -1102,8 +1090,7 @@ public:
       return SWIG_NOWRAP;
     }
 
-    bool make_v2_compatible = v2_compatibility && getCurrentClass() != 0
-      && v2_compat_names_generation;
+    bool make_v2_compatible = old_metatable_bindings && getCurrentClass() && old_compatible_names;
 
     if (make_v2_compatible) {
       // Don't do anything for enums in C mode - they are already
@@ -1178,15 +1165,14 @@ public:
     // this issue to some degree.
     // The idea is the same as in classHandler - to drop old names generation if
     // enum is in class in namespace.
-    const int v2_compat_names_generation_old = v2_compat_names_generation;
-    if (getNSpace() ||
-        ( Getattr(n, "sym:nspace") != 0 && Len(Getattr(n, "sym:nspace")) > 0 ) ) {
-      v2_compat_names_generation = 0;
+    const int old_compatible_names_saved = old_compatible_names;
+    if (getNSpace() || ( Getattr(n, "sym:nspace") != 0 && Len(Getattr(n, "sym:nspace")) > 0 ) ) {
+      old_compatible_names = 0;
     }
     int result = Language::enumDeclaration(n);
     current[STATIC_CONST] = false;
     current[ENUM_CONST] = false;
-    v2_compat_names_generation = v2_compat_names_generation_old;
+    old_compatible_names = old_compatible_names_saved;
     return result;
   }
 
@@ -1334,11 +1320,11 @@ public:
     Setattr(instance_cls, "lua:class_instance:static_hash", static_cls);
     Setattr(static_cls, "lua:class_static:instance_hash", instance_cls);
 
-    const int v2_compat_names_generation_old = v2_compat_names_generation;
+    const int old_compatible_names_saved = old_compatible_names;
     // If class has %nspace enabled, then generation of backward compatible names
     // should be disabled
     if (getNSpace()) {
-      v2_compat_names_generation = 0;
+      old_compatible_names = 0;
     }
 
     /* There is no use for "classes" and "namespaces" arrays. Subclasses are not supported
@@ -1347,7 +1333,7 @@ public:
     // Generate normal wrappers
     Language::classHandler(n);
 
-    v2_compat_names_generation = v2_compat_names_generation_old;
+    old_compatible_names = old_compatible_names_saved;
 
     SwigType_add_pointer(t);
 
@@ -1628,7 +1614,7 @@ public:
     const int result = Language::staticmemberfunctionHandler(n);
     registerMethod(n);
 
-    if (v2_compatibility && result == SWIG_OK && v2_compat_names_generation) {
+    if (old_metatable_bindings && result == SWIG_OK && old_compatible_names) {
       Swig_require("lua_staticmemberfunctionHandler", n, "*lua:name", NIL);
       String *lua_name = Getattr(n, "lua:name");
       // Although this function uses Swig_name_member, it actually generates the Lua name,
@@ -1673,7 +1659,7 @@ public:
 
     if (result == SWIG_OK) {
       // This will add static member variable to the class namespace with name ClassName_VarName
-      if (v2_compatibility && v2_compat_names_generation) {
+      if (old_metatable_bindings && old_compatible_names) {
 	Swig_save("lua_staticmembervariableHandler", n, "lua:name", NIL);
 	String *lua_name = Getattr(n, "lua:name");
 	// Although this function uses Swig_name_member, it actually generates the Lua name,
@@ -1686,7 +1672,7 @@ public:
 	  registerVariable(n, true, getNSpace());
 	}
 	// If static member variable was wrapped as a constant, then
-	// constant wrapper has already performed all actions necessary for v2_compatibility
+	// constant wrapper has already performed all actions necessary for old_metatable_bindings
 	Delete(v2_name);
 	Swig_restore(n);
       }
@@ -1972,6 +1958,10 @@ public:
 
     SetFlag(carrays_hash, "lua:closed");
 
+    // Do arrays describe class instance part or class static part
+    const int is_instance = GetFlag(carrays_hash, "lua:class_instance");
+
+
     String *attr_tab = Getattr(carrays_hash, "attributes");
     Printf(attr_tab, "    {0,0,0}\n};\n");
     Printv(output, attr_tab, NIL);
@@ -1982,7 +1972,17 @@ public:
       Printv(const_tab, tab4, "{LNILKEY, LNILVAL}\n", "};\n", NIL);
     else
       Printf(const_tab, "    {0,0,0,0,0,0}\n};\n");
-    Printv(output, const_tab, NIL);
+    
+    // For the sake of compiling with -Wall -Werror we print constants
+    // only when necessary
+    int need_constants = 0;
+    if ( (elua_ltr || eluac_ltr) && (old_metatable_bindings) )
+      need_constants = 1;
+    else if (!is_instance) // static part need constants tab
+      need_constants = 1;
+
+    if (need_constants)
+      Printv(output, const_tab, NIL);
 
     if (elua_ltr) {
       // Put forward declaration of metatable array
@@ -1991,7 +1991,7 @@ public:
     String *methods_tab = Getattr(carrays_hash, "methods");
     String *metatable_tab_name = Getattr(carrays_hash, "metatable:name");
     if (elua_ltr || eluac_ltr) {
-      if (v2_compatibility)
+      if (old_metatable_bindings)
 	Printv(methods_tab, tab4, "{LSTRKEY(\"const\"), LROVAL(", const_tab_name, ")},\n", NIL);
       if (elua_ltr) {
 	Printv(methods_tab, tab4, "{LSTRKEY(\"__metatable\"), LROVAL(", metatable_tab_name, ")},\n", NIL);
@@ -2023,7 +2023,18 @@ public:
       Printv(output, set_tab, NIL);
     }
 
-    if (!eluac_ltr) {
+    // Heuristic whether we need to print metatable or not.
+    // For the sake of compiling with -Wall -Werror we don't print
+    // metatable for static part.
+    int need_metatable = 0;
+    if (eluac_ltr)
+      need_metatable = 0;
+    else if(!is_instance)
+      need_metatable = 0;
+    else
+      need_metatable = 1;
+
+    if (need_metatable) {
       String *metatable_tab = Getattr(carrays_hash, "metatable");
       assert(metatable_tab);
       if (elua_ltr) {
