@@ -11,13 +11,11 @@
  * Mzscheme language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_mzscheme_cxx[] = "$Id: mzscheme.cxx 13945 2012-12-02 13:37:10Z wsfulton $";
-
 #include "swigmod.h"
 
 #include <ctype.h>
 
-static const char *usage = (char *) "\
+static const char *usage = "\
 Mzscheme Options (available with -mzscheme)\n\
      -declaremodule                         - Create extension that declares a module\n\
      -dynamic-load <library>,[library,...]  - Do not link with these libraries, dynamic load\n\
@@ -38,7 +36,7 @@ static bool declaremodule = false;
 static bool noinit = false;
 static String *load_libraries = NULL;
 static String *module = 0;
-static char *mzscheme_path = (char *) "mzscheme";
+static const char *mzscheme_path = "mzscheme";
 static String *init_func_def = 0;
 
 static File *f_begin = 0;
