@@ -42,7 +42,7 @@
 #include <ctype.h>
 #include <errno.h>
 
-static const char *usage = (char *) "\
+static const char *usage = "\
 PHP Options (available with -php)\n\
      -cppext <ext>    - Change C++ file extension to <ext> (default is cpp)\n\
      -noproxy         - Don't generate proxy classes.\n\
@@ -414,10 +414,9 @@ public:
      * which has to be dynamically generated as it depends on the module name.
      */
     Append(s_header, "#ifdef __GNUC__\n");
-    Append(s_header, "static void SWIG_FAIL() __attribute__ ((__noreturn__));\n");
+    Append(s_header, "static void SWIG_FAIL(TSRMLS_D) __attribute__ ((__noreturn__));\n");
     Append(s_header, "#endif\n\n");
-    Append(s_header, "static void SWIG_FAIL() {\n");
-    Append(s_header, "    TSRMLS_FETCH();\n");
+    Append(s_header, "static void SWIG_FAIL(TSRMLS_D) {\n");
     Append(s_header, "    zend_error(SWIG_ErrorCode(), \"%s\", SWIG_ErrorMsg());\n");
     // zend_error() should never return with the parameters we pass, but if it
     // does, we really don't want to let SWIG_FAIL() return.  This also avoids
@@ -433,8 +432,7 @@ public:
     Printf(s_header, "static void %s_destroy_globals(zend_%s_globals * globals) { (void)globals; }\n", module, module);
 
     Printf(s_header, "\n");
-    Printf(s_header, "static void SWIG_ResetError() {\n");
-    Printf(s_header, "  TSRMLS_FETCH();\n");
+    Printf(s_header, "static void SWIG_ResetError(TSRMLS_D) {\n");
     Printf(s_header, "  SWIG_ErrorMsg() = default_error_msg;\n");
     Printf(s_header, "  SWIG_ErrorCode() = default_error_code;\n");
     Printf(s_header, "}\n");
@@ -445,7 +443,7 @@ public:
     Append(s_header, "  swig_object_wrapper *value;\n");
     Append(s_header, "  int type;\n");
     Append(s_header, "\n");
-    Append(s_header, "  SWIG_ResetError();\n");
+    Append(s_header, "  SWIG_ResetError(TSRMLS_C);\n");
     Append(s_header, "  if(ZEND_NUM_ARGS() != 2 || zend_get_parameters_array_ex(2, args) != SUCCESS) {\n");
     Append(s_header, "    WRONG_PARAM_COUNT;\n");
     Append(s_header, "  }\n");
@@ -460,7 +458,7 @@ public:
     Append(s_header, "  swig_object_wrapper *value;\n");
     Append(s_header, "  int type;\n");
     Append(s_header, "\n");
-    Append(s_header, "  SWIG_ResetError();\n");
+    Append(s_header, "  SWIG_ResetError(TSRMLS_C);\n");
     Append(s_header, "  if(ZEND_NUM_ARGS() != 1 || zend_get_parameters_array_ex(1, args) != SUCCESS) {\n");
     Append(s_header, "    WRONG_PARAM_COUNT;\n");
     Append(s_header, "  }\n");
@@ -736,7 +734,7 @@ public:
 
     Printf(f->code, "SWIG_ErrorCode() = E_ERROR;\n");
     Printf(f->code, "SWIG_ErrorMsg() = \"No matching function for overloaded '%s'\";\n", symname);
-    Printv(f->code, "SWIG_FAIL();\n", NIL);
+    Printv(f->code, "SWIG_FAIL(TSRMLS_C);\n", NIL);
 
     Printv(f->code, "}\n", NIL);
     Wrapper_print(f, s_wrappers);
@@ -854,7 +852,7 @@ public:
 
     // NOTE: possible we ignore this_ptr as a param for native constructor
 
-    Printf(f->code, "SWIG_ResetError();\n");
+    Printf(f->code, "SWIG_ResetError(TSRMLS_C);\n");
 
     if (numopt > 0) {		// membervariable wrappers do not have optional args
       Wrapper_add_local(f, "arg_count", "int arg_count");
@@ -1019,7 +1017,7 @@ public:
     /* Error handling code */
     Printf(f->code, "fail:\n");
     Printv(f->code, cleanup, NIL);
-    Append(f->code, "SWIG_FAIL();\n");
+    Append(f->code, "SWIG_FAIL(TSRMLS_C);\n");
 
     Printf(f->code, "}\n");
 
@@ -2056,7 +2054,6 @@ done:
 
       // Write property SET handlers
       ki = First(shadow_set_vars);
-
       if (ki.key) {
 	// This class has setters.
 	Printf(s_phpclasses, "\n\tfunction __set($var,$value) {\n");
@@ -2079,17 +2076,6 @@ done:
 	  Printf(s_phpclasses, "\t\t$this->%s[$var] = $value;\n", SWIG_DATA);
 	}
 	Printf(s_phpclasses, "\t}\n");
-
-	/* Create __isset for PHP 5.1 and later; PHP 5.0 will just ignore it. */
-	Printf(s_phpclasses, "\n\tfunction __isset($var) {\n");
-	Printf(s_phpclasses, "\t\tif (function_exists('%s_'.$var.'_set')) return true;\n", shadow_classname);
-	Printf(s_phpclasses, "\t\tif ($var === 'thisown') return true;\n");
-	if (baseclass) {
-	  Printf(s_phpclasses, "\t\treturn %s%s::__isset($var);\n", prefix, baseclass);
-	} else {
-	  Printf(s_phpclasses, "\t\treturn array_key_exists($var, $this->%s);\n", SWIG_DATA);
-	}
-	Printf(s_phpclasses, "\t}\n");
       } else {
 	Printf(s_phpclasses, "\n\tfunction __set($var,$value) {\n");
 	Printf(s_phpclasses, "\t\tif ($var === 'thisown') return swig_%s_alter_newobject($this->%s,$value);\n", module, SWIG_PTR);
@@ -2099,18 +2085,10 @@ done:
 	  Printf(s_phpclasses, "\t\t$this->%s[$var] = $value;\n", SWIG_DATA);
 	}
 	Printf(s_phpclasses, "\t}\n");
-	Printf(s_phpclasses, "\n\tfunction __isset($var) {\n");
-	Printf(s_phpclasses, "\t\tif ($var === 'thisown') return true;\n");
-	if (baseclass) {
-	  Printf(s_phpclasses, "\t\treturn %s%s::__isset($var);\n", prefix, baseclass);
-	} else {
-	  Printf(s_phpclasses, "\t\treturn array_key_exists($var, $this->%s);\n", SWIG_DATA);
-	}
-	Printf(s_phpclasses, "\t}\n");
       }
+
       // Write property GET handlers
       ki = First(shadow_get_vars);
-
       if (ki.key) {
 	// This class has getters.
 	Printf(s_phpclasses, "\n\tfunction __get($var) {\n");
@@ -2146,6 +2124,19 @@ done:
 	  Printf(s_phpclasses, "\t\treturn $this->%s[$var];\n", SWIG_DATA);
 	}
 	Printf(s_phpclasses, "\t}\n");
+
+	/* Create __isset for PHP 5.1 and later; PHP 5.0 will just ignore it. */
+	/* __isset() should return true for read-only properties, so check for
+	 * *_get() not *_set(). */
+	Printf(s_phpclasses, "\n\tfunction __isset($var) {\n");
+	Printf(s_phpclasses, "\t\tif (function_exists('%s_'.$var.'_get')) return true;\n", shadow_classname);
+	Printf(s_phpclasses, "\t\tif ($var === 'thisown') return true;\n");
+	if (baseclass) {
+	  Printf(s_phpclasses, "\t\treturn %s%s::__isset($var);\n", prefix, baseclass);
+	} else {
+	  Printf(s_phpclasses, "\t\treturn array_key_exists($var, $this->%s);\n", SWIG_DATA);
+	}
+	Printf(s_phpclasses, "\t}\n");
       } else {
 	Printf(s_phpclasses, "\n\tfunction __get($var) {\n");
 	Printf(s_phpclasses, "\t\tif ($var === 'thisown') return swig_%s_get_newobject($this->%s);\n", module, SWIG_PTR);
@@ -2155,6 +2146,14 @@ done:
 	  Printf(s_phpclasses, "\t\treturn $this->%s[$var];\n", SWIG_DATA);
 	}
 	Printf(s_phpclasses, "\t}\n");
+	Printf(s_phpclasses, "\n\tfunction __isset($var) {\n");
+	Printf(s_phpclasses, "\t\tif ($var === 'thisown') return true;\n");
+	if (baseclass) {
+	  Printf(s_phpclasses, "\t\treturn %s%s::__isset($var);\n", prefix, baseclass);
+	} else {
+	  Printf(s_phpclasses, "\t\treturn array_key_exists($var, $this->%s);\n", SWIG_DATA);
+	}
+	Printf(s_phpclasses, "\t}\n");
       }
 
       if (!class_has_ctor) {
@@ -2274,8 +2273,8 @@ done:
       if (i) {
 	Insert(args, 0, ", ");
       }
-      Printf(director_ctor_code, "} else {\n  %s = (%s *)new SwigDirector_%s(arg0%s TSRMLS_CC);\n}\n", Swig_cresult_name(), ctype, sname, args);
-      Printf(director_prot_ctor_code, "} else {\n  %s = (%s *)new SwigDirector_%s(arg0%s TSRMLS_CC);\n}\n", Swig_cresult_name(), ctype, sname, args);
+      Printf(director_ctor_code, "} else {\n  %s = (%s *)new SwigDirector_%s(arg0 TSRMLS_CC%s);\n}\n", Swig_cresult_name(), ctype, sname, args);
+      Printf(director_prot_ctor_code, "} else {\n  %s = (%s *)new SwigDirector_%s(arg0 TSRMLS_CC%s);\n}\n", Swig_cresult_name(), ctype, sname, args);
       Delete(args);
 
       wrapperType = directorconstructor;
@@ -2336,7 +2335,7 @@ done:
 
     Append(f->code, "return;\n");
     Append(f->code, "fail:\n");
-    Append(f->code, "SWIG_FAIL();\n");
+    Append(f->code, "SWIG_FAIL(TSRMLS_C);\n");
     Printf(f->code, "}\n");
 
     Wrapper_print(f, s_wrappers);
@@ -2387,17 +2386,23 @@ done:
     parms = p;
 
     if (!Getattr(n, "defaultargs")) {
+      // There should always be a "self" parameter first.
+      assert(ParmList_len(parms) > 0);
+
       /* constructor */
       {
 	Wrapper *w = NewWrapper();
 	String *call;
 	String *basetype = Getattr(parent, "classtype");
+
+	// We put TSRMLS_DC after the self parameter in order to cope with
+	// any default parameters.
 	String *target = Swig_method_decl(0, decl, classname, parms, 0, 0);
-	if (((const char *)Char(target))[Len(target) - 2] == '(') {
-	  Insert(target, Len(target) - 1, "TSRMLS_D");
-	} else {
-	  Insert(target, Len(target) - 1, " TSRMLS_DC");
-	}
+	const char * p = Char(target);
+	const char * comma = strchr(p, ',');
+	size_t ins = comma ? comma - p : Len(target) - 1;
+	Insert(target, ins, " TSRMLS_DC");
+
 	call = Swig_csuperclass_call(0, basetype, superparms);
 	Printf(w->def, "%s::%s: %s, Swig::Director(self TSRMLS_CC) {", classname, target, call);
 	Append(w->def, "}");
@@ -2409,12 +2414,14 @@ done:
 
       /* constructor header */
       {
+	// We put TSRMLS_DC after the self parameter in order to cope with
+	// any default parameters.
 	String *target = Swig_method_decl(0, decl, classname, parms, 0, 1);
-	if (((const char *)Char(target))[Len(target) - 2] == '(') {
-	  Insert(target, Len(target) - 1, "TSRMLS_D");
-	} else {
-	  Insert(target, Len(target) - 1, " TSRMLS_DC");
-	}
+	const char * p = Char(target);
+	const char * comma = strchr(p, ',');
+	size_t ins = comma ? comma - p : Len(target) - 1;
+	Insert(target, ins, " TSRMLS_DC");
+
 	Printf(f_directors_h, "    %s;\n", target);
 	Delete(target);
       }
@@ -2700,7 +2707,7 @@ done:
     }
 
     Append(w->code, "fail:\n");
-    Append(w->code, "SWIG_FAIL();\n");
+    Append(w->code, "SWIG_FAIL(TSRMLS_C);\n");
     Append(w->code, "}\n");
 
     // We expose protected methods via an extra public inline method which makes a straight call to the wrapped class' method
