@@ -11,13 +11,11 @@
  * Tcl8 language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tcl8_cxx[] = "$Id: tcl8.cxx 13935 2012-11-24 14:13:58Z wsfulton $";
-
 #include "swigmod.h"
 #include "cparse.h"
 static int treduce = SWIG_cparse_template_reduce(0);
 
-static const char *usage = (char *) "\
+static const char *usage = "\
 Tcl 8 Options (available with -tcl)\n\
      -itcl           - Enable ITcl support\n\
      -nosafe         - Leave out SafeInit module function.\n\
@@ -746,6 +744,7 @@ public:
     have_constructor = 0;
     have_destructor = 0;
     destructor_action = 0;
+    constructor_name = 0;
 
     if (itcl) {
       constructor = NewString("");
@@ -1204,7 +1203,8 @@ public:
       }
     }
 
-    constructor_name = NewString(Getattr(n, "sym:name"));
+    if (!have_constructor)
+      constructor_name = NewString(Getattr(n, "sym:name"));
     have_constructor = 1;
     return SWIG_OK;
   }
